# -*- coding: utf-8 -*-

"""
    Tests for jet algorithms (jetstream_metrics.py & jetstream_metrics_utils)
    Includes:
        – tests for each algorithm defined and expected outputs
        – test for each utility function used in the metric
"""

# imports
import unittest
from jsmetrics.metrics import jet_core_algorithms

# from parameterized import parameterized
import xarray as xr
import numpy as np
from jsmetrics.metrics import (
    jet_core_algorithms_components,
)
from . import set_up_test_uv_data

# docs
__author__ = "Thomas Keel"
__email__ = "thomas.keel.18@ucl.ac.uk"
__status__ = "Development"


class TestKoch2006(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        tested_func = jet_core_algorithms.koch_et_al_2006
        result = tested_func(self.data, ws_threshold=8)
        # check an exact value. Is this necessary?
        self.assertIsInstance(result, xr.Dataset)
        self.assertEqual(float(result["weighted_average_ws"].max()), 8.775158882141113)
        new_data = self.data.isel(plev=0)
        self.assertRaises(ValueError, lambda: tested_func(new_data))

    def test_get_all_hPa_list(self):
        tested_func = jet_core_algorithms_components.get_all_hPa_list
        # make sure it returns an array
        self.assertIsInstance(tested_func(self.data), (np.ndarray))
        # make sure it takes errors wrong types
        # self.assertRaises(TypeError,\
        # lambda: jetstream_metrics_utils.get_all_plev_hPa(['plev']))
        new_data = self.data.rename({"plev": "pl"})
        self.assertRaises(KeyError, lambda: tested_func(new_data))
        new_data2 = self.data.copy()
        new_data2["plev"] = self.data.plev.assign_attrs(units="wrong")
        self.assertRaises(ValueError, lambda: tested_func(new_data2))

    def test_sum_weighted_ws(self):
        tested_func = jet_core_algorithms_components.get_sum_weighted_ws
        self.assertRaises(TypeError, lambda: tested_func(self.data, 1))
        new_data = self.data.rename({"plev": "pl"})
        self.assertRaises(KeyError, lambda: tested_func(new_data, [10, 20]))
        sum_weighted = tested_func(self.data, [0, 100])
        self.assertIsInstance(sum_weighted, xr.DataArray)
        self.assertGreater(sum_weighted.max(), 0)

    def test_weighted_average_ws(self):
        tested_func = jet_core_algorithms_components.get_weighted_average_ws

        self.assertRaises(TypeError, lambda: tested_func(self.data, 1))
        sum_weighted = jet_core_algorithms_components.get_sum_weighted_ws(
            self.data, [0, 100]
        )
        weighted_av = tested_func(sum_weighted, np.array([0, 100]))
        self.assertGreater(weighted_av.min(), 0)
        self.assertGreaterEqual(weighted_av.max(), 0)

        weighted_av_threshold = weighted_av.where(weighted_av >= 30)
        self.assertGreater(weighted_av_threshold.max(), 0)
        self.assertGreaterEqual(weighted_av_threshold.min(), 0)


class TestSchiemann2009(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        tested_func = jet_core_algorithms.schiemann_et_al_2009
        sub_data = self.data.isel(time=slice(0, 1), lon=slice(0, 30))
        result = tested_func(sub_data)
        self.assertTrue(result)


class TestManney2011(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        test_func = jet_core_algorithms.manney_et_al_2011
        # NOTE: this metric is a generator
        subset_data = self.data.sel(plev=slice(25000, 20000)).isel(time=slice(0, 1))
        res = test_func(subset_data.sel(plev=25000))
        self.assertEqual(res["jet_core_id"].max(), 2)


class TestPenaOrtiz2013(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        tested_func = jet_core_algorithms.penaortiz_et_al_2013
        subset_data = self.data.sel(plev=slice(25000, 20000)).isel(time=slice(0, 1))
        res = tested_func(subset_data)
        self.assertTrue("polar_front_jet" in res)
        self.assertEqual(res["subtropical_jet"].max(), 1)

    def test_get_empty_local_wind_maxima(self):
        test_func = jet_core_algorithms_components.get_empty_local_wind_maxima_data
        empty_local_wind_data = test_func(self.data)
        self.assertEqual(empty_local_wind_data.max(), 0)
        self.assertTrue("local_wind_maxima" in empty_local_wind_data)

    def test_get_local_wind_maxima_by_timeunit(self):
        test_func = jet_core_algorithms_components.get_local_wind_maxima_by_timeunit
        self.assertRaises(ValueError, lambda: test_func(self.data.isel(time=0)))


class TestKuang2014(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        tested_func = jet_core_algorithms.kuang_et_al_2014
        lon_data = self.data.sel(plev=50000).isel(time=slice(0, 2))
        result = tested_func(lon_data)
        self.assertEqual(result["jet_ocurrence1_jet_centre2"].max(), 2)
        lon_data = self.data.sel(plev=slice(50000, 25000)).isel(time=slice(0, 2))
        self.assertRaises(ValueError, lambda: tested_func(lon_data))
        plev_data = self.data.sel(plev=slice(25000, 25000)).isel(time=slice(0, 2))
        tested_func(plev_data)


class TestJetStreamCoreIdentificationAlgorithm(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_repr(self):
        tested_alg = jet_core_algorithms_components.JetStreamCoreIdentificationAlgorithm
        test_data = self.data.isel(time=0, lon=0)
        jetCoreAlg = tested_alg(test_data)

        repr(jetCoreAlg)
        jetCoreAlg.run()
        repr(jetCoreAlg)

    def test_ws_thresholds(self):
        tested_alg = jet_core_algorithms_components.JetStreamCoreIdentificationAlgorithm
        self.assertRaises(ValueError, lambda: tested_alg(self.data))
        test_data = self.data.isel(time=0, lon=0)
        self.assertRaises(ValueError, lambda: tested_alg(test_data, -10, 10))
        self.assertRaises(ValueError, lambda: tested_alg(test_data, 10, -10))
        self.assertRaises(ValueError, lambda: tested_alg(test_data, 10, 30))
        self.assertRaises(ValueError, lambda: tested_alg(test_data, 10, 10))

    def test_inner_funcs(self):
        tested_alg = jet_core_algorithms_components.JetStreamCoreIdentificationAlgorithm
        test_data = self.data.isel(time=0, lon=0)
        result = tested_alg(test_data)
        result.run()
        print(result._initial_core_ids.mean())
        self.assertEqual(result._initial_core_ids.mean(), 30.07608695652174)
        self.assertEqual(len(np.where(result._labelled_data["ws"] == "Core")[1]), 46)
        self.assertEqual(
            len(np.where(result._labelled_data["ws"] == "Potential Boundary")[1]),
            81,
        )
        self.assertEqual(result.num_of_cores, 3)
        self.assertListEqual(result.final_jet_cores[0]["index_of_area"][0], [16, 4])

    def test_classmethod(self):
        tested_alg = (
            jet_core_algorithms_components.JetStreamCoreIdentificationAlgorithm.run_algorithm
        )
        test_data = self.data.isel(time=0, lon=0)
        result = tested_alg(test_data)
        self.assertEqual(result._initial_core_ids.mean(), 30.07608695652174)
        self.assertEqual(len(np.where(result._labelled_data["ws"] == "Core")[1]), 46)


class TestJetStreamOccurenceAndCentreAlgorithm(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data().isel(time=slice(0, 1))
        self.tested_alg = (
            jet_core_algorithms_components.JetStreamOccurenceAndCentreAlgorithm
        )

    def test_ws_thresholds(self):
        test_data = self.data.isel(plev=0, time=0)
        self.assertRaises(ValueError, lambda: self.tested_alg(test_data, -10))

    def test_inner_functions(self):
        test_data = self.data.isel(plev=4, time=0)
        result = self.tested_alg(test_data)
        result.run()
        self.assertEqual(float(result._jet_occurence["ws"].max()), 85.84358978271484)
        self.assertListEqual(result._jet_centres[0].tolist(), [0.0, 247.5])

    def test_cls_method(self):
        test_data = self.data.isel(plev=4, time=0)
        result = self.tested_alg.run_algorithm(test_data)
        self.assertEqual(float(result._jet_occurence["ws"].max()), 85.84358978271484)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""The setup script."""
from setuptools import find_packages, setup

NAME = "jsmetrics"
DESCRIPTION = "Library of algorithms and metrics used to characterise and/or identify jet-streams, based on xarray."
URL = "https://github.com/Thomasjkeel/jsmetrics"
AUTHOR = "Thomas Keel"
AUTHOR_EMAIL = "thomas.keel.18@ucl.ac.uk"
REQUIRES_PYTHON = ">=3.7.0"
VERSION = "0.1.0"
LICENSE = "MIT License"

KEYWORDS = "jet-stream climate metrics algorithms xarray"

with open("README.rst") as file:
    readme = file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

# dev_requirements = []
# with open("requirements_dev.txt") as dev:
#     for dependency in dev.readlines():
#         dev_requirements.append(dependency)

requirements = [
    "numpy>=1.21.2",
    "pandas>=0.23",
    "matplotlib>=3.3.2",
    "xarray>=0.19.0",
    "scipy>=1.5.3",
    "dask[array]",
    "netCDF4>=1.5.5.1",
    "bottleneck",
    "cf_xarray",
    "Shapely",
]

dev_requirements = [
    "bump2version",
    "black",
    "pytest",
    "pytest-cov",
    "flake8",
    "parameterized",
    "pre_commit",
    "pylint",
    "pytest",
    "sphinx",
    "numpydoc",
    "sphinx-rtd-theme",
]

setup(
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Atmospheric Science",
    ],
    description=DESCRIPTION,
    python_requires=REQUIRES_PYTHON,
    install_requires=requirements,
    license=LICENSE,
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/x-rst",
    include_package_data=True,
    keywords=KEYWORDS,
    name=NAME,
    packages=find_packages(),
    tests_require=["pytest", "parameterized"],
    extras_require={
        "dev": dev_requirements,
        "data_install": [
            "cdsapi>=0.5.1",
        ],
    },
    url=URL,
    version=VERSION,
    zip_safe=False,
)



# -*- coding: utf-8 -*-

"""
    Jet-stream waviness metrics used in the literature.

    Classes and Functions ordered by paper publish year.
"""

# imports
import xarray
from . import waviness_metrics_components
from jsmetrics.utils import spatial_utils
from jsmetrics.core.check_data import sort_xarray_data_coords

# docs
__author__ = "Thomas Keel"
__email__ = "thomas.keel.18@ucl.ac.uk"
__status__ = "Development"


# @sort_xarray_data_coords(coords=["lat", "lon"])
# def screen_and_simmonds_2013(data):
#     """
#     Slightly adjusted in Screen and Simmonds 2014
#     Method from Screen & Simmonds (2013) https://doi.org/10.1002/grl.50174

#     Parameters
#     ----------
#     data : xarray.Dataset
#         Data containing geopotential height (zg)

#     Returns
#     ----------
#     data : xarray.Dataset
#         Data containing u- and v- component wind speed
#     """
#     if isinstance(data, xarray.DataArray):
#         data = data.to_dataset()
#     return data


# @sort_xarray_data_coords(coords=["lat", "lon"])
# def local_wave_activity(data):
#     """
#     Introduced by Huang and Nakamura for Potential Vorticity, but then used by:
#     Martineau 2017, Chen 2015 and Blackport & Screen 2020 use LWA
#     with 500 hPa zg instead of pv

#     Parameters
#     ----------
#     data : xarray.Dataset
#         Data containing geopotential height (zg)
#     """
#     return data


@sort_xarray_data_coords(coords=["lat", "lon"])
def francis_vavrus_2015(data):
    """
    Calculates the Meridional Circulation Index (MCI). When MCI = 0, the wind is purely zonal, and when MCI= 1 (-1), the flow is
    from the South (North).

    Method from Francis & Vavrus (2015) https://doi.org/10.1088/1748-9326/10/1/014005

    Parameters
    ----------
    data : xarray.Dataset
        Data containing u- and v-component wind

    Returns
    ----------
    data : xarray.Dataset
        Data containing MCI
    """
    #  Step 1. calculating Meridional Circulation Index from data
    data["mci"] = waviness_metrics_components.calc_meridional_circulation_index(data)

    #  Step 2. TODO Calculate anomaly from season
    # maybe TODO: Step 3 Calculate anomaly from season
    return data


@sort_xarray_data_coords(coords=["lat", "lon"])
def cattiaux_et_al_2016(data):
    """
    A sinousity metric for upper-air flow.
    Calculates, for each time unit, the value of the selected isohypse precisely corresponds to the Z500 average over 30–70∘N .
    Then uses the perimeter of this isohype and around 50 .N to calculate sinuosity
    Method from Cattiaux et al (2016) https://doi.org/10.1002/2016GL070309

    NOTE: Currently takes a moderate amount of time i.e. 2 seconds per 100 time unit with 1 plev on AMD Ryzen 5 3600 6-core processor

    Parameters
    ----------
    data : xarray.Dataset
        Data containing geopotential height (zg)

    Returns
    ----------
    data : xarray.Dataset
        Data containing sinuosity of zonal mean by each time unit
    """
    if isinstance(data, xarray.DataArray):
        data = data.to_dataset()

    #  Step 1. get zonal average for each timestep
    if data["time"].size == 1:
        data = data.expand_dims("time")
    data["zonal_mean_zg_30Nto70N"] = (
        data["zg"].sel(lat=slice(30, 70)).groupby("time").mean(...)
    )

    #  Step 2. Get latitude circle of 50 N
    circle_50N = spatial_utils.get_latitude_circle_linestring(50, 0, 360)

    #  Step 3. Loop over each time step and calculate sinuosity
    if data["time"].size == 1:
        # shrink dims again
        data = data.isel(time=0)
        output = waviness_metrics_components.get_sinuosity_of_zonal_mean_zg(
            data, circle_50N
        )
    else:
        output = data.groupby("time").map(
            lambda row: waviness_metrics_components.get_sinuosity_of_zonal_mean_zg(
                row, circle_50N
            )
        )
    return output



# -*- coding: utf-8 -*-

"""
    Operations needed for the jet-stream metrics and jet-stream algorithms that specifically operate on windspeed data.
    Includes the base classes and function for dealing with windspeed vectors and lat/lon or lat/plev slices of windspeed (so called: WindSpeedSlice class)

    Classes and Functions ordered alphabetically.
"""

# imports
import numpy as np
from . import data_utils

# docs
__author__ = "Thomas Keel"
__email__ = "thomas.keel.18@ucl.ac.uk"
__status__ = "Development"


def get_resultant_wind(u, v):
    """
    Gets wind vector from u-wind and v-wind
    """
    return np.sqrt(u**2 + v**2)


def get_wind_direction_in_degrees(u, v):
    """
    Gets wind direction from u-wind and v-wind
    In degrees (0-360)
    """
    return (180 + (180 / np.pi) * np.arctan2(u, v)) % 360


def get_zonal_mean(data):
    """
    Will get the zonal mean either by pressure level (plev) or for one layer

    Parameters
    ----------
    data : xarray.Dataset
        Data containing lon and plev coords

    Returns
    ----------
    zonal_mean : xarray.DataSet
        zonal mean data

    Raises
    ----------
    KeyError
        when 'lon' not discovered as coord
    """
    if "lon" not in data.coords:
        raise KeyError("data does not contain 'lon' coord")

    coords_for_mean = ["lon", "plev"]
    if "plev" not in data.coords or int(data["plev"].count()) == 1:
        coords_for_mean = ["lon"]
    zonal_mean = data.mean(coords_for_mean)
    return zonal_mean


class WindSpeedSlice:
    """
    Base class for windspeed slice.

    Methods
    -------
    label_slice:
        Label the windspeed slice using where condtion
    get_values:
        Get values using xarray
    """

    def __init__(self, data, req_variables=("ua", "va")):
        # these will check that the correct data variables/coords
        # are available in the xarray data input
        self.req_variables = req_variables
        self._check_input_data_can_be_used_for_windspeed_slice(data)
        self.values = self._calc_windspeed(data)
        self.values = self.values.rename("ws").to_dataset()

    def __init_subclass__(cls, req_coords, *a, **kw):
        cls.req_coords = req_coords

    def __getitem__(self, item):
        return self.values[item]

    @staticmethod
    def _calc_windspeed(data):
        """
        The reason this exists is that their may be a point in time where
        constraitns on this calculation will need to be made
        i.e. if too much data or too much RAM in use
        """
        return get_resultant_wind(data["ua"], data["va"])

    def _check_input_data_can_be_used_for_windspeed_slice(self, data):
        """
        Check input data is valid for the windspeed slice.
        """
        data_utils.check_if_data_is_xarray_datatype(data)
        data_utils.check_coords_in_data(data, self.req_coords)
        data_utils.check_variables_in_data(data, self.req_variables)
        data_utils.remove_unwanted_coords_from_data(
            data, wanted_coords=self.req_coords, unwanted_coords=("bnds",)
        )

    def label_slice(self, condition, label):
        """
        Returns labels where condition is met.
        """
        return self.values.where(condition, other=label)

    def get_values(self):
        """
        Get values.
        """
        return self.values


class PressureLevelWindSpeedSlice(WindSpeedSlice, req_coords=("lat", "lon")):
    """
    Data will be lon*lat

    Methods
    -------
    label_slice:
        Label the windspeed slice using where condtion
    get_values:
        Get values using xarray
    """


class LatitudeWindSpeedSlice(WindSpeedSlice, req_coords=("lat", "plev")):
    """
    Data will be lon*plev

    Methods
    -------
    label_slice:
        Label the windspeed slice using where condtion
    get_values:
        Get values using xarray
    """



# imports
import unittest
from jsmetrics.metrics import waviness_metrics
from . import set_up_test_uv_data, set_up_test_zg_data

# docs
__author__ = "Thomas Keel"
__email__ = "thomas.keel.18@ucl.ac.uk"
__status__ = "Development"


class TestFrancisVavrus2015(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_uv_data()

    def test_metric(self):
        result = waviness_metrics.francis_vavrus_2015(self.data)
        self.assertEqual(float(result["mci"].mean()), -0.01847001537680626)
        self.assertTrue(result["mci"].max() == 1)
        self.assertTrue(result["mci"].min() == -1)


# class TestLocalWaveActivity(unittest.TestCase):
#     def setUp(self):
#         self.data = set_up_test_zg_data()

#     def test_metric(self):
#         result = waviness_metrics.local_wave_activity(self.data)
#         self.assertTrue(result)


class TestCattiaux2016(unittest.TestCase):
    def setUp(self):
        self.data = set_up_test_zg_data()

    def test_metric(self):
        tested_func = waviness_metrics.cattiaux_et_al_2016
        subset_data = self.data.isel(plev=0)
        res = tested_func(subset_data)
        tested_func(subset_data["zg"])
        self.assertEqual(round(float(res["sinuosity"].max()), 3), 2.749)



# -*- coding: utf-8 -*-

"""
    Various utility functions needed for the jet-stream metrics and algorithms not belonging to windspeed or spatial utils.
    The module is built from xarray data-structures, so this file contains all the stuff that helps the library handle xarray dataset/dataarrays

    Classes and Functions ordered alphabetically
"""

# imports
import itertools
import numpy as np
import xarray as xr
import scipy.signal

# docs
__author__ = "Thomas Keel"
__email__ = "thomas.keel.18@ucl.ac.uk"
__status__ = "Development"


def check_at_least_n_plevs_in_data(data, n_plevs):
    """
    Checks there are at least two pressure-levels (plevs) in xarray dataset

    Parameters
    ----------
    data : xarray.Dataset
        Data to check

    Raises
    ----------
    ValueError :
        If not two pressure levels
    """
    if data["plev"].count() < n_plevs:
        raise ValueError(
            "Need at least %s pressure levels (plevs) for calculation" % (n_plevs)
        )


def check_coords_in_data(data, req_coords):
    """
    Will check if a given coordinates are in the data and raises a key error if any of them are not.
    This function is needed to check before algorithm continues and uses too much memory.
    Built from xarray.

    Parameters
    ----------
    data : xarray.Dataset
        Data to check
    req_coords : array-like or tuple
        Coordinates to check if in data

    Raises
    ----------
    KeyError :
        If any given coord is not in data
    """
    for coord in req_coords:
        if coord not in data.coords:
            raise KeyError("'%s' is not in the data" % (coord,))


def check_if_data_is_xarray_datatype(data):
    """
    What it says on the tin.

    Parameters
    ----------
    data : xarray.Dataset
        Data to check

    Raises
    ----------
    TypeError :
        If input is not an xarray data type
    """
    if not isinstance(data, xr.Dataset) or isinstance(data, xr.DataArray):
        raise TypeError("input needs to be xarray.DataSet or xarray.DataArray")


def check_variables_in_data(data, req_variables):
    """
    What it says on the tin.
    Built from xarray

    Parameters
    ----------
    data : xarray.Dataset
        Data to check
    req_variables : array-like
        Variables needed in data

    Raises
    ----------
    KeyError :
        If variables not in data
    """
    for var in req_variables:
        if var not in data.variables:
            raise KeyError("'%s' is not the data" % (var,))


def get_local_maxima(arr, axis=0):
    """
    Uses scipy.signal.argrelextrema to get index location of minimum value in array

    Taken from
    https://stackoverflow.com/questions/4624970/finding-local-maxima-minima-with-numpy-in-a-1d-numpy-array

    Parameters
    ----------
    arr : array-like
        array to find index location of minima value from
    axis : int
        axis for scipy.signal.argrelextrema
    """
    return scipy.signal.argrelextrema(arr, np.greater, axis=axis)


def get_local_minima(arr, axis=0):
    """
    Uses scipy.signal.argrelextrema to get index location of maximum value in array

    Taken from
    https://stackoverflow.com/questions/4624970/finding-local-maxima-minima-with-numpy-in-a-1d-numpy-array

    Parameters
    ----------
    arr : array-like
        array to find index location of maxima value from
    axis : int
        axis for scipy.signal.argrelextrema
    """
    return scipy.signal.argrelextrema(arr, np.less, axis=axis)


def get_num_of_decimal_places(num):
    """
    Gets number of decimal places in a float

    Parameters
    ----------
    num : float or int
        input number to get decimal places from

    Returns
    ----------
    decimal_places : int
        number of decimal places
    """
    num = "{:f}".format(num).rstrip("0")
    decimal_places = num[::-1].find(".")
    return decimal_places


def remove_duplicates(arr):
    """
    Removes duplicates from array. From:
    https://stackoverflow.com/questions/2213923/removing-duplicates-from-a-list-of-lists

    Parameters
    ----------
    arr : array-like
        arr to remove duplicates from

    Returns
    ----------
    arr : array-like
        arr with no duplicates
    """

    arr.sort()
    return list(v for v, _ in itertools.groupby(arr))


def remove_unwanted_coords_from_data(data, wanted_coords, unwanted_coords=()):
    """
    What it says on the tin.
    Built from xarray

    Parameters
    ----------
    data : xarray.Dataset
        Data to check

    wanted_coords : array-like or tuple
        Coords to retain in data
    unwanted_coords : array-like or tuple
        Coords to remove from data


    Raises
    ----------
    ValueError :
        If coord cannot be removed from data
    """
    dims = set(data.dims)
    for rem in unwanted_coords:
        try:
            dims.remove(rem)
        except Exception as e:
            # a little sloppy, but probably okay
            e

    wanted_coords = set(wanted_coords)
    difference = dims.difference(set(wanted_coords))
    if len(difference) > 0:
        raise ValueError(
            "Unwanted coords in data: %s.\
             Please subset/remove so that the slice can be 2D."
            % (difference,)
        )


def rescale_lat_resolution(lats, lat_resolution):
    """
    Component of method from Barnes & Polvani (2013) https://doi.org/10.1175/JCLI-D-12-00536.1
    & Grise & Polvani 2017 https://doi.org/10.1175/JCLI-D-16-0849.1
    & Bracegirdle et al (2018) https://doi.org/10.1175/JCLI-D-17-0320.1


    Parameters
    ----------
    lats : xr.DataArray or array-like
        Array of latitude values
    lat_resolution : int or float
        Latitude resolution in degrees

    Returns
    ----------
    output : numpy.array
        Rescaled array of latitude values
    """
    return np.arange(min(lats), max(lats) + lat_resolution, lat_resolution)
